/*
 *  AscBitmaps.c
 *  by Kirk Abbott and Ben Allan
 *  Created: 1/94
 *  Version: $Revision: 1.7 $
 *  Version control file: $RCSfile: AscBitmaps.c,v $
 *  Date last modified: $Date: 1998/04/25 18:14:13 $
 *  Last modified by: $Author: ballan $
 *
 *  This file is part of the ASCEND Tcl/Tk interface
 *
 *  Copyright 1997, Carnegie Mellon University
 *
 *  The ASCEND Tcl/Tk interface is free software; you can redistribute
 *  it and/or modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  The ASCEND Tcl/Tk interface is distributed in hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include <tcl.h>
#include <tk.h>
#include "config.h"
#include "AscBitmaps.h"

#define BITSTRING static unsigned char
#define BITDATA(s) ((char *)&((s)[0]))
int Asc_RegisterBitmaps(Tcl_Interp *interp)
{
  BITSTRING sqr1x1_bits[] = { 0x01 };
  BITSTRING sqr2x2_bits[] = { 0x03, 0x03 };
  BITSTRING sqr3x3_bits[] = { 0x07, 0x07, 0x07 };
  BITSTRING sqr3x3h_bits[] = { 0x07, 0x05, 0x07 };
  BITSTRING x3x3_bits[] = { 0x05, 0x02, 0x05 };
  BITSTRING sqr4x4_bits[] = { 0x0f, 0x0f, 0x0f, 0x0f };
  BITSTRING sqr4x4h_bits[] = { 0x0f, 0x09, 0x09, 0x0f };
  BITSTRING x4x4_bits[] = { 0x09, 0x06, 0x06, 0x09 };
  BITSTRING sqr5x5_bits[] = { 0x1f, 0x1f, 0x1f, 0x1f, 0x1f };
  BITSTRING sqrx5x5_bits[] = { 0x1f, 0x1b, 0x15, 0x0b, 0x1f };
  BITSTRING sqr5x5h_bits[] = { 0x1f, 0x11, 0x11, 0x11, 0x1f };
  BITSTRING x5x5_bits[] = { 0x11, 0x0a, 0x04, 0x0a, 0x11 };
  BITSTRING sqr6x6_bits[] = { 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f };
  BITSTRING sqrx6x6_bits[] = { 0x3f, 0x33, 0x2d, 0x2d, 0x33, 0x3f };
  BITSTRING sqr6x6h_bits[] = { 0x3f, 0x21, 0x21, 0x21, 0x21, 0x3f };
  BITSTRING x6x6_bits[] = { 0x21, 0x12, 0x0c, 0x0c, 0x12, 0x21 };
  BITSTRING sqr7x7_bits[] = { 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f };
  BITSTRING sqrx7x7_bits[] = { 0x7f, 0x63, 0x55, 0x49, 0x55, 0x63, 0x7f };
  BITSTRING sqr7x7h_bits[] = { 0x7f, 0x41, 0x41, 0x41, 0x41, 0x41, 0x7f };
  BITSTRING x7x7_bits[] = { 0x41, 0x22, 0x14, 0x08, 0x14, 0x22, 0x41 };
  BITSTRING sqr8x8_bits[] = { 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,0xff};
  BITSTRING sqrx8x8_bits[] = { 0xff, 0xc3, 0xa5, 0x99, 0x99, 0xa5,0xc3,0xff};
  BITSTRING sqr8x8h_bits[] = { 0xff, 0x81, 0x81, 0x81, 0x81, 0x81,0x81,0xff};
  BITSTRING x8x8_bits[] = { 0x81, 0x42, 0x24, 0x18, 0x18, 0x24, 0x42, 0x81};
  BITSTRING sqr9x9_bits[] = {
    0xff, 0x01, 0xff, 0x01, 0xff, 0x01, 0xff, 0x01, 0xff,
    0x01, 0xff, 0x01, 0xff, 0x01, 0xff, 0x01, 0xff, 0x01 };
  BITSTRING sqrx9x9_bits[] = {
    0xff, 0x01, 0x83, 0x01, 0x45, 0x01, 0x29, 0x01, 0x11,
    0x01, 0x29, 0x01, 0x45, 0x01, 0x83, 0x01, 0xff, 0x01 };
  BITSTRING sqr9x9h_bits[] = {
    0xff, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
    0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0xff, 0x01 };
  BITSTRING x9x9_bits[] = {
    0x01, 0x01, 0x82, 0x00, 0x44, 0x00, 0x28, 0x00, 0x10,
    0x00, 0x28, 0x00, 0x44, 0x00, 0x82, 0x00, 0x01, 0x01 };

  BITSTRING sqr10x10_bits[] = {
    0xff, 0x03, 0xff, 0x03, 0xff, 0x03, 0xff, 0x03, 0xff, 0x03, 0xff, 0x03,
    0xff, 0x03, 0xff, 0x03, 0xff, 0x03, 0xff, 0x03 };
  BITSTRING sqrx10x10_bits[] = {
    0xff, 0x03, 0x03, 0x03, 0x85, 0x02, 0x49, 0x02, 0x31, 0x02, 0x31, 0x02,
    0x49, 0x02, 0x85, 0x02, 0x03, 0x03, 0xff, 0x03 };
  BITSTRING sqr10x10h_bits[] = {
    0xff, 0x03, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0x01, 0x02,
    0x01, 0x02, 0x01, 0x02, 0x01, 0x02, 0xff, 0x03 };
  BITSTRING x10x10_bits[] = {
    0x01, 0x02, 0x02, 0x01, 0x84, 0x00, 0x48, 0x00, 0x30, 0x00, 0x30, 0x00,
    0x48, 0x00, 0x84, 0x00, 0x02, 0x01, 0x01, 0x02 };

  BITSTRING sqr11x11_bits[] = {
    0xff, 0x07, 0xff, 0x07, 0xff, 0x07, 0xff, 0x07, 0xff, 0x07, 0xff, 0x07,
    0xff, 0x07, 0xff, 0x07, 0xff, 0x07, 0xff, 0x07, 0xff, 0x07 };
  BITSTRING sqrx11x11_bits[] = {
    0xff, 0x07, 0x03, 0x06, 0x05, 0x05, 0x89, 0x04, 0x51, 0x04, 0x21, 0x04,
    0x51, 0x04, 0x89, 0x04, 0x05, 0x05, 0x03, 0x06, 0xff, 0x07 };
  BITSTRING sqr11x11h_bits[] = {
    0xff, 0x07, 0x01, 0x04, 0x01, 0x04, 0x01, 0x04, 0x01, 0x04, 0x01, 0x04,
    0x01, 0x04, 0x01, 0x04, 0x01, 0x04, 0x01, 0x04, 0xff, 0x07 };
  BITSTRING x11x11_bits[] = {
    0x01, 0x04, 0x02, 0x02, 0x04, 0x01, 0x88, 0x00, 0x50, 0x00, 0x20, 0x00,
    0x50, 0x00, 0x88, 0x00, 0x04, 0x01, 0x02, 0x02, 0x01, 0x04 };

  BITSTRING sqr12x12_bits[] = {
    0xff, 0x0f, 0xff, 0x0f, 0xff, 0x0f, 0xff, 0x0f, 0xff, 0x0f, 0xff, 0x0f,
    0xff, 0x0f, 0xff, 0x0f, 0xff, 0x0f, 0xff, 0x0f, 0xff, 0x0f, 0xff, 0x0f };
  BITSTRING sqrx12x12_bits[] = {
    0xff, 0x0f, 0x03, 0x0c, 0x05, 0x0a, 0x09, 0x09, 0x91, 0x08, 0x61, 0x08,
    0x61, 0x08, 0x91, 0x08, 0x09, 0x09, 0x05, 0x0a, 0x03, 0x0c, 0xff, 0x0f };
  BITSTRING sqr12x12h_bits[] = {
    0xff, 0x0f, 0x01, 0x08, 0x01, 0x08, 0x01, 0x08, 0x01, 0x08, 0x01, 0x08,
    0x01, 0x08, 0x01, 0x08, 0x01, 0x08, 0x01, 0x08, 0x01, 0x08, 0xff, 0x0f };
  BITSTRING x12x12_bits[] = {
    0x01, 0x08, 0x02, 0x04, 0x04, 0x02, 0x08, 0x01, 0x90, 0x00, 0x60, 0x00,
    0x60, 0x00, 0x90, 0x00, 0x08, 0x01, 0x04, 0x02, 0x02, 0x04, 0x01, 0x08 };

  BITSTRING sqr13x13_bits[] = {
    0xff, 0x1f, 0xff, 0x1f, 0xff, 0x1f, 0xff, 0x1f, 0xff, 0x1f, 0xff, 0x1f,
    0xff, 0x1f, 0xff, 0x1f, 0xff, 0x1f, 0xff, 0x1f, 0xff, 0x1f, 0xff, 0x1f,
    0xff, 0x1f };
  BITSTRING sqrx13x13_bits[] = {
    0xff, 0x1f, 0x03, 0x18, 0x05, 0x14, 0x09, 0x12, 0x11, 0x11, 0xa1, 0x10,
    0x41, 0x10, 0xa1, 0x10, 0x11, 0x11, 0x09, 0x12, 0x05, 0x14, 0x03, 0x18,
    0xff, 0x1f };
  BITSTRING sqr13x13h_bits[] = {
    0xff, 0x1f, 0x01, 0x10, 0x01, 0x10, 0x01, 0x10, 0x01, 0x10, 0x01, 0x10,
    0x01, 0x10, 0x01, 0x10, 0x01, 0x10, 0x01, 0x10, 0x01, 0x10, 0x01, 0x10,
    0xff, 0x1f };
  BITSTRING x13x13_bits[] = {
    0x01, 0x10, 0x02, 0x08, 0x04, 0x04, 0x08, 0x02, 0x10, 0x01, 0xa0, 0x00,
    0x40, 0x00, 0xa0, 0x00, 0x10, 0x01, 0x08, 0x02, 0x04, 0x04, 0x02, 0x08,
    0x01, 0x10 };

  BITSTRING sqr14x14_bits[] = {
    0xff, 0x3f, 0xff, 0x3f, 0xff, 0x3f, 0xff, 0x3f, 0xff, 0x3f, 0xff, 0x3f,
    0xff, 0x3f, 0xff, 0x3f, 0xff, 0x3f, 0xff, 0x3f, 0xff, 0x3f, 0xff, 0x3f,
    0xff, 0x3f, 0xff, 0x3f };
  BITSTRING sqrx14x14_bits[] = {
    0xff, 0x3f, 0x03, 0x30, 0x05, 0x28, 0x09, 0x24, 0x11, 0x22, 0x21, 0x21,
    0xc1, 0x20, 0xc1, 0x20, 0x21, 0x21, 0x11, 0x22, 0x09, 0x24, 0x05, 0x28,
    0x03, 0x30, 0xff, 0x3f };
  BITSTRING sqr14x14h_bits[] = {
    0xff, 0x3f, 0x01, 0x20, 0x01, 0x20, 0x01, 0x20, 0x01, 0x20, 0x01, 0x20,
    0x01, 0x20, 0x01, 0x20, 0x01, 0x20, 0x01, 0x20, 0x01, 0x20, 0x01, 0x20,
    0x01, 0x20, 0xff, 0x3f };
  BITSTRING x14x14_bits[] = {
    0x01, 0x20, 0x02, 0x10, 0x04, 0x08, 0x08, 0x04, 0x10, 0x02, 0x20, 0x01,
    0xc0, 0x00, 0xc0, 0x00, 0x20, 0x01, 0x10, 0x02, 0x08, 0x04, 0x04, 0x08,
    0x02, 0x10, 0x01, 0x20 };

  BITSTRING toolAttributes_bits[] = {
   0xff, 0xff, 0x01, 0x80, 0x01, 0x80, 0xf9, 0x9f, 0xf9, 0x9f, 0x01, 0x80,
   0x01, 0x80, 0xf9, 0x9f, 0xf9, 0x9f, 0x01, 0x80, 0x01, 0x80, 0xf9, 0x9f,
   0xf9, 0x9f, 0x01, 0x80, 0x01, 0x80, 0xff, 0xff};
  BITSTRING stop_bits[] = {
    0xf0, 0x0f, 0xf8, 0x1f, 0x0c, 0x30, 0x06, 0x60, 0x13, 0xc8, 0x23, 0xc4,
    0x43, 0xc2, 0x83, 0xc1, 0x83, 0xc1, 0x43, 0xc2, 0x23, 0xc4, 0x13, 0xc8,
    0x06, 0x60, 0x0c, 0x30, 0xf8, 0x1f, 0xf0, 0x0f };

  BITSTRING feet_bits[]= {
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x78, 0x00, 0x48, 0x00,
    0x48, 0x00, 0x88, 0x00, 0x88, 0x00, 0x88, 0x07, 0x08, 0x3a, 0x08, 0xd5,
    0x88, 0x8a, 0x18, 0x80, 0x10, 0xc0, 0xf0, 0x7f};

  BITSTRING wfeet_bits[] = {
    0x00, 0x00, 0xfe, 0x00, 0x01, 0x01, 0x7e, 0x01, 0x01, 0x01, 0x7c, 0x01,
    0x42, 0x01, 0x88, 0x00, 0x88, 0x00, 0x88, 0x07, 0x08, 0x3a, 0x08, 0xd5,
    0x88, 0x8a, 0x18, 0x80, 0x10, 0xc0, 0xf0, 0x7f};

  BITSTRING hourglass_bits[] = {
    0xff, 0xff, 0x07, 0x55, 0x55, 0x05, 0xa2, 0x2a, 0x03, 0x66, 0x15, 0x01,
    0xa2, 0x2a, 0x03, 0x66, 0x15, 0x01, 0xc2, 0x0a, 0x03, 0x46, 0x05, 0x01,
    0x82, 0x0a, 0x03, 0x06, 0x05, 0x01, 0x02, 0x03, 0x03, 0x86, 0x05, 0x01,
    0xc2, 0x0a, 0x03, 0x66, 0x15, 0x01, 0xa2, 0x2a, 0x03, 0x66, 0x15, 0x01,
    0xa2, 0x2a, 0x03, 0x66, 0x15, 0x01, 0xa2, 0x2a, 0x03, 0xff, 0xff, 0x07,
    0xab, 0xaa, 0x02};

  BITSTRING grablock_bits[] = {
   0x00, 0x00, 0x00, 0x00, 0xfe, 0x00, 0x00, 0x00, 0x00, 0xfe, 0x00, 0x00,
   0x00, 0x00, 0xfe, 0x00, 0xe0, 0x0f, 0x00, 0xfe, 0x00, 0x70, 0x1c, 0x00,
   0xfe, 0x00, 0x18, 0x30, 0x00, 0xfe, 0x00, 0x0c, 0x60, 0x00, 0xfe, 0x00,
   0x04, 0x40, 0x00, 0xfe, 0x00, 0x06, 0xc0, 0x00, 0xfe, 0x00, 0x06, 0xc0,
   0x00, 0xfe, 0x00, 0x06, 0xc0, 0x00, 0xfe, 0x00, 0x06, 0xc0, 0x00, 0xfe,
   0x00, 0xfe, 0xff, 0x00, 0xfe, 0x00, 0xfe, 0xff, 0x00, 0xfe, 0x00, 0xfe,
   0xff, 0x00, 0xfe, 0x00, 0x7e, 0xfc, 0x00, 0xfe, 0x00, 0x7e, 0xfc, 0x00,
   0xfe, 0x00, 0x7e, 0xfc, 0x00, 0xfe, 0x00, 0xfe, 0xfe, 0x00, 0xfe, 0x00,
   0xfe, 0xfe, 0x00, 0xfe, 0x00, 0xfe, 0xfe, 0x00, 0xfe, 0x00, 0x7e, 0xfc,
   0x00, 0xfe, 0x00, 0x3e, 0xf8, 0x00, 0xfe, 0x00, 0xfe, 0xff, 0x00, 0xfe,
   0x00, 0x00, 0x00, 0x00, 0xfe, 0x00, 0x00, 0x00, 0x00, 0xfe, 0x00, 0x00,
   0x00, 0x00, 0xfe};

  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_1"),BITDATA(sqr1x1_bits),1,1);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_h1"),BITDATA(sqr1x1_bits),1,1);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_x1"),BITDATA(sqr1x1_bits),1,1);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_c1"),BITDATA(sqr1x1_bits),1,1);

  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_2"),BITDATA(sqr2x2_bits),2,2);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_h2"),BITDATA(sqr2x2_bits),2,2);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_x2"),BITDATA(sqr2x2_bits),2,2);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_c2"),BITDATA(sqr2x2_bits),2,2);

  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_3"),BITDATA(sqr3x3_bits),3,3);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_x3"),BITDATA(sqr3x3_bits),3,3);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_h3"),BITDATA(sqr3x3h_bits),3,3);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_c3"),BITDATA(x3x3_bits),3,3);

  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_4"),BITDATA(sqr4x4_bits),4,4);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_x4"),BITDATA(sqr4x4_bits),4,4);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_h4"),BITDATA(sqr4x4h_bits),4,4);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_c4"),BITDATA(x4x4_bits),4,4);

  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_5"),BITDATA(sqr5x5_bits),5,5);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_x5"),BITDATA(sqrx5x5_bits),5,5);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_h5"),BITDATA(sqr5x5h_bits),5,5);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_c5"),BITDATA(x5x5_bits),5,5);

  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_6"),BITDATA(sqr6x6_bits),6,6);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_x6"),BITDATA(sqrx6x6_bits),6,6);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_h6"),BITDATA(sqr6x6h_bits),6,6);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_c6"),BITDATA(x6x6_bits),6,6);

  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_7"),BITDATA(sqr7x7_bits),7,7);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_x7"),BITDATA(sqrx7x7_bits),7,7);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_h7"),BITDATA(sqr7x7h_bits),7,7);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_c7"),BITDATA(x7x7_bits),7,7);

  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_8"),BITDATA(sqr8x8_bits),8,8);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_x8"),BITDATA(sqrx8x8_bits),8,8);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_h8"),BITDATA(sqr8x8h_bits),8,8);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_c8"),BITDATA(x8x8_bits),8,8);

  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_9"),BITDATA(sqr9x9_bits),9,9);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_x9"),BITDATA(sqrx9x9_bits),9,9);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_h9"),BITDATA(sqr9x9h_bits),9,9);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_c9"),BITDATA(x9x9_bits),9,9);

  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_10"),BITDATA(sqr10x10_bits),10,10);
  Tk_DefineBitmap(interp,
                  Tk_GetUid("asc_sq_x10"),BITDATA(sqrx10x10_bits),10,10);
  Tk_DefineBitmap(interp,
                  Tk_GetUid("asc_sq_h10"),BITDATA(sqr10x10h_bits),10,10);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_c10"),BITDATA(x10x10_bits),10,10);

  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_11"),BITDATA(sqr11x11_bits),11,11);
  Tk_DefineBitmap(interp,
                  Tk_GetUid("asc_sq_x11"),BITDATA(sqrx11x11_bits),11,11);
  Tk_DefineBitmap(interp,
                  Tk_GetUid("asc_sq_h11"),BITDATA(sqr11x11h_bits),11,11);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_c11"),BITDATA(x11x11_bits),11,11);

  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_12"),BITDATA(sqr12x12_bits),12,12);
  Tk_DefineBitmap(interp,
                  Tk_GetUid("asc_sq_x12"),BITDATA(sqrx12x12_bits),12,12);
  Tk_DefineBitmap(interp,
                  Tk_GetUid("asc_sq_h12"),BITDATA(sqr12x12h_bits),12,12);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_c12"),BITDATA(x12x12_bits),12,12);

  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_13"),BITDATA(sqr13x13_bits),13,13);
  Tk_DefineBitmap(interp,
                  Tk_GetUid("asc_sq_x13"),BITDATA(sqrx13x13_bits),13,13);
  Tk_DefineBitmap(interp,
                  Tk_GetUid("asc_sq_h13"),BITDATA(sqr13x13h_bits),13,13);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_c13"),BITDATA(x13x13_bits),13,13);

  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_14"),BITDATA(sqr14x14_bits),14,14);
  Tk_DefineBitmap(interp,
                  Tk_GetUid("asc_sq_x14"),BITDATA(sqrx14x14_bits),14,14);
  Tk_DefineBitmap(interp,
                  Tk_GetUid("asc_sq_h14"),BITDATA(sqr14x14h_bits),14,14);
  Tk_DefineBitmap(interp,Tk_GetUid("asc_sq_c14"),BITDATA(x14x14_bits),14,14);

  Tk_DefineBitmap(interp,Tk_GetUid("toolAttributes"),
                  BITDATA(toolAttributes_bits),16,16);
  Tk_DefineBitmap(interp,Tk_GetUid("stop"),BITDATA(stop_bits),16,16);
  Tk_DefineBitmap(interp,Tk_GetUid("feet"),BITDATA(feet_bits),16,16);
  Tk_DefineBitmap(interp,Tk_GetUid("wfeet"),BITDATA(wfeet_bits),16,16);
  Tk_DefineBitmap(interp,Tk_GetUid("hourglass"),BITDATA(hourglass_bits),16,16);
  Tk_DefineBitmap(interp,Tk_GetUid("grablock"),BITDATA(grablock_bits),33,27);
  return 0;
}
/* and for those of you who still dont get it, try thinking toolless 1993 */
